==> uxkit-0.0.2.tgz: package/built/UXKit/TableView.js <==
/// <reference path="../../typings/tsd.d.ts"/>
import * as React from "react";
import * as RN from "react-native";
import { View } from "./View";
import { ScrollView } from "./ScrollView";
import { TableViewCellReusingValues } from "./TableViewCell";
import { TableViewStyle } from "./Enum";
/**
 * TableView
 */
export class TableView extends ScrollView {
    constructor(props) {
        super(props);
        this.reuseImp = new TableViewReuseImp(this);
        this.sectionHeaderImp = new TableViewSectionHeaderImp(this);
    }
    reloadData() {
        this.reuseImp.reloadData();
    }
    componentDidMount() {
        super.componentDidMount();
        this.reloadData();
    }
    contentHeight() {
        return this.reuseImp.contentHeight;
    }
    content() {
        let cellElements = [];
        for (let reuseIdentifier in this.reuseImp.requestReuseIdentifiers()) {
            let type = this.reuseImp.requestReuseIdentifiers()[reuseIdentifier];
            let c = { type: type };
            for (let index = 0; index < TableViewReuseImp.maxLength; index++) {
                let reusingValue = this.reuseImp.requestReusingValue(reuseIdentifier, index);
                cellElements.push(React.createElement(RN.Animated.View, {key: reuseIdentifier + index, style: { position: "absolute", left: 0, right: 0, top: reusingValue.top, height: reusingValue.height }}, React.createElement(c.type, {tableView: this, reusingValue: reusingValue})));
            }
        }
        return (React.createElement(RN.View, {style: { height: this.contentHeight() }}, this.tableHeaderView(), cellElements, this.tableFooterView(), this.sectionHeaderImp.requestSectionHeaderElements()));
    }
    tableHeaderView() {
        if (!!this.props.tableHeaderView) {
            return (React.createElement(View, {frame: { x: 0, mx: 0, y: 0 }}, this.props.tableHeaderView));
        }
        else {
            return null;
        }
    }
    tableFooterView() {
        if (!!this.props.tableFooterView) {
            return (React.createElement(View, {frame: { x: 0, mx: 0, my: 0 }}, this.props.tableFooterView));
        }
        else {
            return null;
        }
    }
    overlayer() {
        if (this.props.tableViewStyle == TableViewStyle.Plain) {
            return this.sectionHeaderImp.requestSectionHeaderOverlayerElements();
        }
        else {
            return null;
        }
    }
    onScroll() {
        super.onScroll();
        this.reuseImp.resetReuseValues();
        if (this.props.tableViewStyle == TableViewStyle.Plain) {
            this.sectionHeaderImp.resetSectionHeaderValues();
        }
    }
}
TableView.defaultProps = Object.assign({}, ScrollView.defaultProps, {
    tableViewStyle: TableViewStyle.Grouped,
    rowHeight: 44.0,
    allowsSelection: true,
    allowsEditing: false,
});
/**
 * TableViewSectionHeaderImp
 */
export class TableViewSectionHeaderImp {
    constructor(tableView) {
        this.sectionHeaderBounces = [];
        this.sectionHeaderHeights = [];
        // Plain TableView
        this.sectionHeaderValues = [];
        this.sectionHeaderOpacitys = [];
        this.sectionLayoutAreas = [];
        this.tableView = tableView;
    }
    reloadData(sectionHeaderBounces, sectionHeaderHeights) {
        this.sectionHeaderBounces = sectionHeaderBounces;
        this.sectionHeaderHeights = sectionHeaderHeights;
        if (this.tableView.props.tableViewStyle == TableViewStyle.Plain) {
            this.createSectionHeaderValues();
        }
    }
    requestSectionHeaderElements() {
        let elements = [];
        for (let section = 0; section < this.sectionHeaderHeights.length; section++) {
            let sectionHeight = this.sectionHeaderHeights[section];
            let sectionHeaderView = this.tableView.props.delegate.viewForHeaderInSection !== undefined ? this.tableView.props.delegate.viewForHeaderInSection(this.tableView, section) : null;
            elements.push(React.createElement(RN.View, {key: "sectionHeader" + section, style: { position: "absolute", left: 0, right: 0, top: this.sectionHeaderBounces[section], height: sectionHeight, backgroundColor: this.tableView.props.backgroundColor }}, sectionHeaderView));
        }
        return (React.createElement(RN.View, {style: { position: "absolute", left: 0, right: 0, top: 0, bottom: 0 }, pointerEvents: "none"}, elements));
    }
    createSectionHeaderValues() {
        this.sectionHeaderValues = [];
        this.sectionLayoutAreas = [];
        for (let index = 0; index < this.sectionHeaderBounces.length; index++) {
            let headerPosition = this.sectionHeaderBounces[index];
            this.sectionHeaderValues.push(new RN.Animated.Value(0.0));
            this.sectionHeaderOpacitys.push(new RN.Animated.Value(0.0));
        }
        for (var index = 0; index < this.sectionHeaderBounces.length; index++) {
            let staticArea = this.sectionHeaderBounces[index];
            let changeArea = this.sectionHeaderBounces[index + 1] - this.sectionHeaderHeights[index];
            if (isNaN(changeArea)) {
                this.sectionLayoutAreas.push(staticArea);
            }
            else {
                this.sectionLayoutAreas.push(staticArea, changeArea);
            }
        }
    }
    resetSectionHeaderValues() {
        if (this.tableView.props.tableViewStyle !== TableViewStyle.Plain) {
            return;
        }
        let currentY = this.tableView.contentOffset.y;
        let targetIndex = -1;
        for (var index = 0; index < this.sectionLayoutAreas.length; index++) {
            var area = this.sectionLayoutAreas[index];
            if (currentY < area) {
                break;
            }
            else {
                targetIndex = index;
            }
        }
        if (targetIndex === -1) {
            for (var index = 0; index < this.sectionHeaderValues.length; index++) {
                this.sectionHeaderOpacitys[index].setValue(0.0);
            }
        }
        else if (targetIndex % 2 === 1) {
            // change
            let prevSection = Math.floor(targetIndex / 2);
            let nextSection = Math.floor(targetIndex / 2) + 1;
            let offsetA = currentY - this.sectionLayoutAreas[targetIndex];
            let offsetB = this.sectionLayoutAreas[nextSection] - currentY;
            this.sectionHeaderValues[prevSection].setValue(-offsetA);
            this.sectionHeaderValues[nextSection].setValue(offsetB);
            for (var index = 0; index < this.sectionHeaderValues.length; index++) {
                if (index != prevSection && index != nextSection) {
                    this.sectionHeaderOpacitys[index].setValue(0.0);
                }
                else if (index != nextSection) {
                    this.sectionHeaderOpacitys[index].setValue(1.0);
                }
            }
        }
        else {
            // static
            let currentSection = Math.floor(targetIndex / 2);
            this.sectionHeaderValues[currentSection].setValue(0.0);
            for (var index = 0; index < this.sectionHeaderValues.length; index++) {
                if (index != currentSection) {
                    this.sectionHeaderOpacitys[index].setValue(0.0);
                }
                else {
                    this.sectionHeaderOpacitys[index].setValue(1.0);
                }
            }
        }
    }
    requestSectionHeaderOverlayerElements() {
        let elements = [];
        for (let section = 0; section < this.sectionHeaderValues.length; section++) {
            let sectionHeight = this.tableView.props.delegate.heightForHeaderInSection !== undefined ? this.tableView.props.delegate.heightForHeaderInSection(this.tableView, section) : 0.0;
            let sectionHeaderView = this.tableView.props.delegate.viewForHeaderInSection !== undefined ? this.tableView.props.delegate.viewForHeaderInSection(this.tableView, section) : null;
            let animatedValue = this.sectionHeaderValues[section];
            let alphaValue = this.sectionHeaderOpacitys[section];
            elements.push(React.createElement(RN.Animated.View, {key: "sectionHeader" + section, style: { position: "absolute", opacity: alphaValue, left: 0, right: 0, top: animatedValue, height: sectionHeight, backgroundColor: this.tableView.props.backgroundColor }}, sectionHeaderView));
        }
        elements.reverse();
        return (React.createElement(RN.View, {style: { position: "absolute", left: 0, right: 0, top: 0, bottom: 0 }, pointerEvents: "none"}, elements));
    }
}
/**
 * TableViewReuseImp
 */
export class TableViewReuseImp {
    constructor(tableView) {
        this.reuseIdentifiers = {};
        this.cellValues = [];
        this.reusingValues = {};
        this.scrollBounces = [];
        this.prevLayoutY = -1;
        this.nextLayoutY = -1;
        this.tableView = tableView;
    }
    get contentHeight() {
        if (this.scrollBounces.length > 0) {
            let lastRowBounce = this.scrollBounces[this.scrollBounces.length - 1];
            return lastRowBounce;
        }
        return 0;
    }
    reloadData() {
        this.reuseIdentifiers = {};
        this.cellValues = [];
        this.scrollBounces = [];
        this.prevLayoutY = -1;
        this.nextLayoutY = -1;
        let sectionResult = this.createCellValues();
        this.createReusingValues();
        this.bindReusingValuesToCellValues();
        this.resetReuseValues();
        this.tableView.sectionHeaderImp.reloadData(sectionResult.sectionHeaderBounces, sectionResult.sectionHeaderHeights);
        this.tableView.forceUpdate();
    }
    createCellValues() {
        let sectionHeaderBounces = [];
        let sectionHeaderHeights = [];
        let pos = 0;
        if (!!this.tableView.props.tableHeaderView && typeof this.tableView.props.tableHeaderView.props.style.height === "number") {
            pos += this.tableView.props.tableHeaderView.props.style.height;
        }
        for (let section = 0; section < this.tableView.props.dataSource.numberOfSectionsInTableView(this.tableView); section++) {
            // section header injector <<<
            sectionHeaderBounces.push(pos);
            let headerHeight = this.tableView.props.delegate.heightForHeaderInSection !== undefined ? this.tableView.props.delegate.heightForHeaderInSection(this.tableView, section) : 0.0;
            sectionHeaderHeights.push(headerHeight);
            pos += headerHeight;
            // EOF
            let rowCount = this.tableView.props.dataSource.numberOfRowsInSection(this.tableView, section);
            for (let row = 0; row < rowCount; row++) {
                let rowHeight = (this.tableView.props.delegate && this.tableView.props.delegate.heightForRowAtIndexPath) ? this.tableView.props.delegate.heightForRowAtIndexPath(this.tableView, { section: section, row: row }) : this.tableView.props.rowHeight;
                let rowArgument = this.tableView.props.dataSource.cellForRowAtIndexPath(this.tableView, { section: section, row: row });
                this.reuseIdentifiers[rowArgument.reuseIdentifier] = rowArgument.cellClass;
                if (true) {
                    let value = new TableViewCellReusingValues();
                    value.topValue = pos;
                    value.heightValue = rowHeight;
                    value.otherValues.indexPath = { section: section, row: row };
                    value.otherValues.arguments = rowArgument;
                    this.cellValues.push(value);
                }
                this.scrollBounces.push(pos);
                pos += rowHeight;
            }
        }
        this.scrollBounces.push(pos);
        if (!!this.tableView.props.tableFooterView && typeof this.tableView.props.tableFooterView.props.style.height === "number") {
            pos += this.tableView.props.tableFooterView.props.style.height;
            this.scrollBounces.push(pos);
        }
        return { sectionHeaderBounces: sectionHeaderBounces, sectionHeaderHeights: sectionHeaderHeights };
    }
    createReusingValues() {
        for (let reuseIdentifier in this.reuseIdentifiers) {
            if (this.reusingValues[reuseIdentifier] && this.reusingValues[reuseIdentifier].length > 0) {
                continue;
            }
            this.reusingValues[reuseIdentifier] = [];
            for (let index = 0; index < TableViewReuseImp.maxLength; index++) {
                this.reusingValues[reuseIdentifier].push(new TableViewCellReusingValues());
            }
        }
    }
    bindReusingValuesToCellValues() {
        for (let reuseIdentifier in this.reuseIdentifiers) {
            let curr = 0;
            this.cellValues.forEach(cellValue => {
                let reusingValue = this.reusingValues[reuseIdentifier][curr];
                if (reusingValue === undefined) {
                    curr = 0;
                    reusingValue = this.reusingValues[reuseIdentifier][0];
                }
                cellValue.other = reusingValue.other;
                cellValue.top = reusingValue.top;
                cellValue.height = reusingValue.height;
                curr++;
            });
        }
    }
    resetReuseValues() {
        let currentY = this.tableView.contentOffset.y;
        if (currentY < this.nextLayoutY && currentY > this.prevLayoutY) {
            return;
        }
        let nowIdx = 0;
        this.scrollBounces.forEach(element => {
            if (element < currentY) {
                nowIdx++;
            }
        });
        this.prevLayoutY = this.scrollBounces[nowIdx - 1];
        this.nextLayoutY = this.scrollBounces[nowIdx + 1];
        if (nowIdx > 3) {
            nowIdx -= 3;
        }
        else {
            nowIdx = 0;
        }
        this.cellValues.forEach((value, idx) => {
            if (idx < nowIdx) {
                return;
            }
            if (idx >= nowIdx + TableViewReuseImp.maxLength) {
                return;
            }
            if (value.other.indexPath === value.otherValues.indexPath && value.other.reusingCell && value.other.reusingCell.loaded) {
                return;
            }
            value.other.indexPath = value.otherValues.indexPath;
            value.other.arguments = value.otherValues.arguments;
            value.top.setValue(value.topValue);
            value.height.setValue(value.heightValue);
            if (value.other.reusingCell) {
                value.other.reusingCell.reload();
            }
        });
    }
    requestReuseIdentifiers() {
        return this.reuseIdentifiers;
    }
    requestReusingValue(reuseIdentifier, index) {
        return this.reusingValues[reuseIdentifier][index];
    }
    requestCell(indexPath) {
        for (var reuseIdentifier in this.reusingValues) {
            let items = this.reusingValues[reuseIdentifier];
            for (var index = 0; index < items.length; index++) {
                var element = items[index];
                if (element.other.indexPath && element.other.indexPath.section === indexPath.section && element.other.indexPath.row === indexPath.row) {
                    return element.other.reusingCell;
                }
            }
        }
        return undefined;
    }
}
TableViewReuseImp.maxLength = 20;

==> uxkit-0.0.3.tgz: package/built/UXKit/TableView.js <==
/// <reference path="../../typings/tsd.d.ts"/>
import * as React from "react";
import * as RN from "react-native";
import { ViewController } from "./ViewController";
import { View } from "./View";
import { ScrollView } from "./ScrollView";
import { TableViewCellReusingValues } from "./TableViewCell";
import { TableViewStyle } from "./Enum";
/**
 * TableView
 */
export class TableView extends ScrollView {
    constructor(props) {
        super(props);
        this.firstLoaded = false;
        this.batchLoading = false;
        this.reuseImp = new TableViewReuseImp(this);
        this.sectionHeaderImp = new TableViewSectionHeaderImp(this);
    }
    reloadData() {
        if (!this.firstLoaded) {
            ViewController.firstQueue.push((completionBlock) => {
                this.reuseImp.reloadData(completionBlock);
                this.firstLoaded = true;
            });
        }
        else {
            if (this.batchLoading) {
                return;
            }
            this.batchLoading = true;
            RN.InteractionManager.runAfterInteractions(() => {
                this.reuseImp.reloadData();
                this.batchLoading = false;
            });
        }
    }
    componentDidMount() {
        super.componentDidMount();
        this.reloadData();
    }
    contentHeight() {
        return this.reuseImp.contentHeight;
    }
    content() {
        let cellElements = [];
        for (let reuseIdentifier in this.reuseImp.requestReuseIdentifiers()) {
            let type = this.reuseImp.requestReuseIdentifiers()[reuseIdentifier];
            let c = { type: type };
            for (let index = 0; index < TableViewReuseImp.maxLength; index++) {
                let reusingValue = this.reuseImp.requestReusingValue(reuseIdentifier, index);
                cellElements.push(React.createElement(RN.Animated.View, {key: reuseIdentifier + index, style: { position: "absolute", left: 0, right: 0, top: reusingValue.top, height: reusingValue.height }}, React.createElement(c.type, {tableView: this, reusingValue: reusingValue})));
            }
        }
        return (React.createElement(RN.View, {style: { height: this.contentHeight() }}, this.tableHeaderView(), cellElements, this.tableFooterView(), this.sectionHeaderImp.requestSectionHeaderElements()));
    }
    tableHeaderView() {
        if (!!this.props.tableHeaderView) {
            return (React.createElement(View, {frame: { x: 0, mx: 0, y: 0 }}, this.props.tableHeaderView));
        }
        else {
            return null;
        }
    }
    tableFooterView() {
        if (!!this.props.tableFooterView) {
            return (React.createElement(View, {frame: { x: 0, mx: 0, my: 0 }}, this.props.tableFooterView));
        }
        else {
            return null;
        }
    }
    overlayer() {
        if (this.props.tableViewStyle == TableViewStyle.Plain) {
            return this.sectionHeaderImp.requestSectionHeaderOverlayerElements();
        }
        else {
            return null;
        }
    }
    onScroll() {
        super.onScroll();
        this.reuseImp.resetReuseValues();
        if (this.props.tableViewStyle == TableViewStyle.Plain) {
            this.sectionHeaderImp.resetSectionHeaderValues();
        }
    }
}
TableView.defaultProps = Object.assign({}, ScrollView.defaultProps, {
    tableViewStyle: TableViewStyle.Grouped,
    rowHeight: 44.0,
    allowsSelection: true,
    allowsEditing: false,
});
/**
 * TableViewSectionHeaderImp
 */
export class TableViewSectionHeaderImp {
    constructor(tableView) {
        this.sectionHeaderBounces = [];
        this.sectionHeaderHeights = [];
        // Plain TableView
        this.sectionHeaderValues = [];
        this.sectionHeaderOpacitys = [];
        this.sectionLayoutAreas = [];
        this.tableView = tableView;
    }
    reloadData(sectionHeaderBounces, sectionHeaderHeights) {
        this.sectionHeaderBounces = sectionHeaderBounces;
        this.sectionHeaderHeights = sectionHeaderHeights;
        if (this.tableView.props.tableViewStyle == TableViewStyle.Plain) {
            this.createSectionHeaderValues();
        }
    }
    requestSectionHeaderElements() {
        let elements = [];
        for (let section = 0; section < this.sectionHeaderHeights.length; section++) {
            let sectionHeight = this.sectionHeaderHeights[section];
            let sectionHeaderView = this.tableView.props.delegate.viewForHeaderInSection !== undefined ? this.tableView.props.delegate.viewForHeaderInSection(this.tableView, section) : null;
            elements.push(React.createElement(RN.View, {key: "sectionHeader" + section, style: { position: "absolute", left: 0, right: 0, top: this.sectionHeaderBounces[section], height: sectionHeight, backgroundColor: this.tableView.props.backgroundColor }}, sectionHeaderView));
        }
        return (React.createElement(RN.View, {style: { position: "absolute", left: 0, right: 0, top: 0, bottom: 0 }, pointerEvents: "none"}, elements));
    }
    createSectionHeaderValues() {
        this.sectionHeaderValues = [];
        this.sectionLayoutAreas = [];
        for (let index = 0; index < this.sectionHeaderBounces.length; index++) {
            let headerPosition = this.sectionHeaderBounces[index];
            this.sectionHeaderValues.push(new RN.Animated.Value(0.0));
            this.sectionHeaderOpacitys.push(new RN.Animated.Value(0.0));
        }
        for (var index = 0; index < this.sectionHeaderBounces.length; index++) {
            let staticArea = this.sectionHeaderBounces[index];
            let changeArea = this.sectionHeaderBounces[index + 1] - this.sectionHeaderHeights[index];
            if (isNaN(changeArea)) {
                this.sectionLayoutAreas.push(staticArea);
            }
            else {
                this.sectionLayoutAreas.push(staticArea, changeArea);
            }
        }
    }
    resetSectionHeaderValues() {
        if (this.tableView.props.tableViewStyle !== TableViewStyle.Plain) {
            return;
        }
        let currentY = this.tableView.contentOffset.y;
        let targetIndex = -1;
        for (var index = 0; index < this.sectionLayoutAreas.length; index++) {
            var area = this.sectionLayoutAreas[index];
            if (currentY < area) {
                break;
            }
            else {
                targetIndex = index;
            }
        }
        if (targetIndex === -1) {
            for (var index = 0; index < this.sectionHeaderValues.length; index++) {
                this.sectionHeaderOpacitys[index].setValue(0.0);
            }
        }
        else if (targetIndex % 2 === 1) {
            // change
            let prevSection = Math.floor(targetIndex / 2);
            let nextSection = Math.floor(targetIndex / 2) + 1;
            let offsetA = currentY - this.sectionLayoutAreas[targetIndex];
            let offsetB = this.sectionLayoutAreas[nextSection] - currentY;
            this.sectionHeaderValues[prevSection].setValue(-offsetA);
            this.sectionHeaderValues[nextSection].setValue(offsetB);
            for (var index = 0; index < this.sectionHeaderValues.length; index++) {
                if (index != prevSection && index != nextSection) {
                    this.sectionHeaderOpacitys[index].setValue(0.0);
                }
                else if (index != nextSection) {
                    this.sectionHeaderOpacitys[index].setValue(1.0);
                }
            }
        }
        else {
            // static
            let currentSection = Math.floor(targetIndex / 2);
            this.sectionHeaderValues[currentSection].setValue(0.0);
            for (var index = 0; index < this.sectionHeaderValues.length; index++) {
                if (index != currentSection) {
                    this.sectionHeaderOpacitys[index].setValue(0.0);
                }
                else {
                    this.sectionHeaderOpacitys[index].setValue(1.0);
                }
            }
        }
    }
    requestSectionHeaderOverlayerElements() {
        let elements = [];
        for (let section = 0; section < this.sectionHeaderValues.length; section++) {
            let sectionHeight = this.tableView.props.delegate.heightForHeaderInSection !== undefined ? this.tableView.props.delegate.heightForHeaderInSection(this.tableView, section) : 0.0;
            let sectionHeaderView = this.tableView.props.delegate.viewForHeaderInSection !== undefined ? this.tableView.props.delegate.viewForHeaderInSection(this.tableView, section) : null;
            let animatedValue = this.sectionHeaderValues[section];
            let alphaValue = this.sectionHeaderOpacitys[section];
            elements.push(React.createElement(RN.Animated.View, {key: "sectionHeader" + section, style: { position: "absolute", opacity: alphaValue, left: 0, right: 0, top: animatedValue, height: sectionHeight, backgroundColor: this.tableView.props.backgroundColor }}, sectionHeaderView));
        }
        elements.reverse();
        return (React.createElement(RN.View, {style: { position: "absolute", left: 0, right: 0, top: 0, bottom: 0 }, pointerEvents: "none"}, elements));
    }
}
/**
 * TableViewReuseImp
 */
export class TableViewReuseImp {
    constructor(tableView) {
        this.reuseIdentifiers = {};
        this.cellValues = [];
        this.reusingValues = {};
        this.scrollBounces = [];
        this.prevLayoutY = -1;
        this.nextLayoutY = -1;
        this.tableView = tableView;
    }
    get contentHeight() {
        if (this.scrollBounces.length > 0) {
            let lastRowBounce = this.scrollBounces[this.scrollBounces.length - 1];
            return lastRowBounce;
        }
        return 0;
    }
    reloadData(completionBlock = undefined) {
        this.reuseIdentifiers = {};
        this.cellValues = [];
        this.scrollBounces = [];
        this.prevLayoutY = -1;
        this.nextLayoutY = -1;
        let sectionResult = this.createCellValues();
        this.createReusingValues();
        this.bindReusingValuesToCellValues();
        this.resetReuseValues();
        this.tableView.sectionHeaderImp.reloadData(sectionResult.sectionHeaderBounces, sectionResult.sectionHeaderHeights);
        this.tableView.forceUpdate(() => {
            setTimeout(completionBlock, 300);
        });
    }
    createCellValues() {
        let sectionHeaderBounces = [];
        let sectionHeaderHeights = [];
        let pos = 0;
        if (!!this.tableView.props.tableHeaderView && typeof this.tableView.props.tableHeaderView.props.style.height === "number") {
            pos += this.tableView.props.tableHeaderView.props.style.height;
        }
        for (let section = 0; section < this.tableView.props.dataSource.numberOfSectionsInTableView(this.tableView); section++) {
            // section header injector <<<
            sectionHeaderBounces.push(pos);
            let headerHeight = this.tableView.props.delegate.heightForHeaderInSection !== undefined ? this.tableView.props.delegate.heightForHeaderInSection(this.tableView, section) : 0.0;
            sectionHeaderHeights.push(headerHeight);
            pos += headerHeight;
            // EOF
            let rowCount = this.tableView.props.dataSource.numberOfRowsInSection(this.tableView, section);
            for (let row = 0; row < rowCount; row++) {
                let rowHeight = (this.tableView.props.delegate && this.tableView.props.delegate.heightForRowAtIndexPath) ? this.tableView.props.delegate.heightForRowAtIndexPath(this.tableView, { section: section, row: row }) : this.tableView.props.rowHeight;
                let rowArgument = this.tableView.props.dataSource.cellForRowAtIndexPath(this.tableView, { section: section, row: row });
                this.reuseIdentifiers[rowArgument.reuseIdentifier] = rowArgument.cellClass;
                if (true) {
                    let value = new TableViewCellReusingValues();
                    value.topValue = pos;
                    value.heightValue = rowHeight;
                    value.otherValues.indexPath = { section: section, row: row };
                    value.otherValues.arguments = rowArgument;
                    this.cellValues.push(value);
                }
                this.scrollBounces.push(pos);
                pos += rowHeight;
            }
        }
        this.scrollBounces.push(pos);
        if (!!this.tableView.props.tableFooterView && typeof this.tableView.props.tableFooterView.props.style.height === "number") {
            pos += this.tableView.props.tableFooterView.props.style.height;
            this.scrollBounces.push(pos);
        }
        return { sectionHeaderBounces: sectionHeaderBounces, sectionHeaderHeights: sectionHeaderHeights };
    }
    createReusingValues() {
        for (let reuseIdentifier in this.reuseIdentifiers) {
            if (this.reusingValues[reuseIdentifier] && this.reusingValues[reuseIdentifier].length > 0) {
                continue;
            }
            this.reusingValues[reuseIdentifier] = [];
            for (let index = 0; index < TableViewReuseImp.maxLength; index++) {
                this.reusingValues[reuseIdentifier].push(new TableViewCellReusingValues());
            }
        }
    }
    bindReusingValuesToCellValues() {
        for (let reuseIdentifier in this.reuseIdentifiers) {
            let curr = 0;
            this.cellValues.forEach(cellValue => {
                let reusingValue = this.reusingValues[reuseIdentifier][curr];
                if (reusingValue === undefined) {
                    curr = 0;
                    reusingValue = this.reusingValues[reuseIdentifier][0];
                }
                cellValue.other = reusingValue.other;
                cellValue.top = reusingValue.top;
                cellValue.height = reusingValue.height;
                curr++;
            });
        }
    }
    resetReuseValues() {
        let currentY = this.tableView.contentOffset.y;
        if (currentY < this.nextLayoutY && currentY > this.prevLayoutY) {
            return;
        }
        let nowIdx = 0;
        this.scrollBounces.forEach(element => {
            if (element < currentY) {
                nowIdx++;
            }
        });
        this.prevLayoutY = this.scrollBounces[nowIdx - 1];
        this.nextLayoutY = this.scrollBounces[nowIdx + 1];
        if (nowIdx > 3) {
            nowIdx -= 3;
        }
        else {
            nowIdx = 0;
        }
        this.cellValues.forEach((value, idx) => {
            if (idx < nowIdx) {
                return;
            }
            if (idx >= nowIdx + TableViewReuseImp.maxLength) {
                return;
            }
            if (value.other.indexPath === value.otherValues.indexPath && value.other.reusingCell && value.other.reusingCell.loaded) {
                return;
            }
            value.other.indexPath = value.otherValues.indexPath;
            value.other.arguments = value.otherValues.arguments;
            value.top.setValue(value.topValue);
            value.height.setValue(value.heightValue);
            if (value.other.reusingCell) {
                value.other.reusingCell.reload();
            }
        });
    }
    requestReuseIdentifiers() {
        return this.reuseIdentifiers;
    }
    requestReusingValue(reuseIdentifier, index) {
        return this.reusingValues[reuseIdentifier][index];
    }
    requestCell(indexPath) {
        for (var reuseIdentifier in this.reusingValues) {
            let items = this.reusingValues[reuseIdentifier];
            for (var index = 0; index < items.length; index++) {
                var element = items[index];
                if (element.other.indexPath && element.other.indexPath.section === indexPath.section && element.other.indexPath.row === indexPath.row) {
                    return element.other.reusingCell;
                }
            }
        }
        return undefined;
    }
}
TableViewReuseImp.maxLength = 20;
